// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; indent-tabs-mode: nil; -*-
////////////////////////////////////////////////////////////////////////
// File: timedgate.h
// This file contains sample implementation for the test generation:
// Train gate controller with delayed stop implementation under test.
// Authors:
//   Marius Mikucionis marius@cs.aau.dk
////////////////////////////////////////////////////////////////////////

#ifndef TIMEDGATE_H
#define TIMEDGATE_H

#include "tron/adapter.h"
#include "tron/Executable.h"

#include <pthread.h>
#include <semaphore.h>


class GateAdapter;
class TrainQueue;

// LateGate maintains the incomming train list.
// the train is stopped if another is passing the critical region.
// the stopped train is let to go if the passage is free.
class Gate: protected Executable
{
public:
    Gate(int id, GateAdapter*);
    virtual ~Gate();
    virtual void start();
    virtual void dumpState();
protected:
    GateAdapter* ga;// shared data among all gates/threads
    struct timespec timeout;
    int id;
    volatile enum { SID_FREE, SID_APPR, SID_RED, SID_HOLD,
                    SID_GREEN, SID_WAIT, SID_REMOVE} state;
    virtual void state_free();
    virtual void state_appr();
    virtual void state_red();
    virtual void state_hold();
    virtual void state_green();
    virtual void state_wait();
    virtual void state_remove();
    TrainQueue *tqueue;
    volatile bool abort; // signal to abort testing

    bool sendOutputOn(struct timespec &timeout,
                      int32_t chan, int16_t n, const int32_t data[]);
    virtual void* threadExecute();
    pthread_cond_t dead_c;
};

class TrainQueue
{
public:
    TrainQueue(GateAdapter *ta);
    virtual ~TrainQueue();
    virtual int add(int id); // id >= 0, returns size
    virtual void rem(int id);

protected:
    struct node {
        int id; node* next;
        node(int _id): id(_id), next(NULL) {}
    };
    node *volatile head, *volatile tail;
    volatile int size;
    pthread_mutex_t list_m; // for thread safe queue locking
    GateAdapter *adapter;
};

class GateAdapter: public TestAdapter, protected Executable
{
public:
    GateAdapter(Reporter* rep, int railways, int mutant=0);
    virtual ~GateAdapter();
/* TestAdapter part: */
    void start_imp();
    void perform_imp(int32_t chan, uint16_t n, const int32_t data[]);

/* AbstractTime part: */
    virtual void* threadExecute();

/* GateAdapter (additional, very specific) part: */
    void locked_perform(int32_t chan, uint16_t n, const int32_t data[]);
    void reportOutput(int32_t chan, uint16_t n, const int32_t data[]){
        rep->report_now(rep, chan, n, data);
    }
protected:
    friend class Gate;
    int gatecnt;
    Gate **gates;
    TrainQueue* tqueue;
};

#endif
